/**
 * Routes for the Vue router.
 *
 * Part of the basebox sample TODO app.
 * https://basebox.tech
 */

import { store } from "../store";
import { createRouter, createWebHistory } from 'vue-router'
import HomeView from '../views/HomeView.vue'
import OAUthError from "../components/OAUthError.vue";
import { objectToQueryString } from "../util/net";

const router = createRouter({
  history: createWebHistory(import.meta.env.BASE_URL),
  routes: [
    {
      path: '/',
      name: 'home',
      component: HomeView
    },
    {
      path: '/about',
      name: 'about',
      // route level code-splitting
      // this generates a separate chunk (About.[hash].js) for this route
      // which is lazy-loaded when the route is visited.
      component: () => import('../views/AboutView.vue')
    },
    {
      path: '/oauth-error',
      name: 'oauth-error',
      component: OAUthError,
      props: true,
    },
  ]
})

router.beforeEach(async (to, from) => {

  // Handle OAuth callback request.
  // This request is coming in after the user entered her/his credentials at the IdP
  // login form; the IdP (e.g. Keycloak) redirects the browser to the URL of this route.
  if (to.path === "/oauth-callback") {
    let queryString = objectToQueryString(to.query);
    console.info(`Got /oauth-callback with query string '${queryString}`);
    /* Pass the query parameters to the broker to complete login. We will get
     * the session token and user data in return.
     *
     * The URL we post to is broker's base URL and the `openid_connect_path`
     * setting from broker's config.
     */
    const response = await fetch(`${store.baseboxHost}/oauth/complete-login?${queryString}`, {
      method: "POST",
    });
    if (!response.ok) {
      /* redirect to error component */
      console.error("Failed to complete login/get session data: " + response.statusText);
      return {
        name: 'oauth-error',
        query: {
          code: response.status,
          addlInfo: await response.text()
        }
      }
    }

    /* store session data; this is a JSON object of the following form:
      {
        "token":"47caa9df-ac89-45a8-8b22-9c0925d6aed0",
        "username":"tester",
        "first_name":"Fred",
        "last_name":"Feuerstein",
        "roles":[
          "/user"
        ],
        "claims":{
          "groups":[
            "/user"
          ]
        }
      }
    */
    store.session = await response.json();
    store.userName = store.session.first_name ? store.session.first_name : store.session.username;

    /* redirect to home */
    return { name: 'home' };
  }

});

export default router
